mod calendar;
mod config;
mod pollen;
mod weather;

use calendar::fetch_next_events;
use chrono::{DateTime, TimeZone, Utc};
use chrono_tz::Tz;
use config::Config;
use pollen::fetch_pollen_api;
use weather::{fetch_weather, geocode_city, WeatherSummary};

#[tokio::main]
async fn main() {
    let config = match Config::load() {
        Ok(cfg) => cfg,
        Err(e) => {
            eprintln!("Failed to load config: {e}");
            std::process::exit(1);
        }
    };

    let weather: Option<WeatherSummary> =
        match fetch_weather(&config.location, &config.weather_api_key).await {
            Ok(data) => Some(data),
            Err(e) => {
                eprintln!("Failed to fetch weather: {e}");
                None
            }
        };
    let (_lat, _lon) = if weather.is_some() {
        match geocode_city(&config.location, &config.weather_api_key).await {
            Ok((lat, lon)) => (lat, lon),
            Err(_) => (0.0, 0.0),
        }
    } else {
        (0.0, 0.0)
    };
    if let Some(w) = &weather {
        if w.obs_time_unix > 0 {
            match config.timezone.parse::<Tz>() {
                Ok(tz) => {
                    let dt_local = tz.from_utc_datetime(
                        &Utc.timestamp_opt(w.obs_time_unix, 0).unwrap().naive_utc(),
                    );
                    println!(
                        "Weather (as of {}):",
                        dt_local.format("%Y-%m-%d %H:%M:%S %Z")
                    );
                }
                Err(_) => {
                    println!(
                        "Weather (as of unix timestamp {}) (timezone parse error)",
                        w.obs_time_unix
                    );
                }
            }
        } else {
            println!("Weather:");
        }
        println!(" - Temperature: {}", w.temp);
        println!(" - Conditions: {}", w.current_desc);
        println!(" - High: {}, Low: {}", w.high, w.low);
        println!(" - Forecast: {}", w.daily_desc);
    } else {
        println!("Weather: N/A");
    }

    match fetch_pollen_api(&config.pollen_zip).await {
        Ok(p) => {
            if let Ok(dt) = DateTime::parse_from_rfc3339(&p.forecast_date) {
                match config.timezone.parse::<Tz>() {
                    Ok(tz) => {
                        let local_time = dt.with_timezone(&tz);
                        println!(
                            "\nPollen.com ({})\n  Forecast for {}:",
                            p.location,
                            local_time.format("%Y-%m-%d %H:%M:%S %Z")
                        );
                    }
                    Err(_) => {
                        println!(
                            "\nPollen.com ({})\n  Forecast for {} (timezone parse error):",
                            p.location, p.forecast_date
                        );
                    }
                }
            } else {
                println!(
                    "\nPollen.com ({})\n  Forecast for {}:",
                    p.location, p.forecast_date
                );
            }

            for day in &["Yesterday", "Today", "Tomorrow"] {
                if let Some(period) = p.periods.get(*day) {
                    println!(
                        "    {:9}: {:>4.1} ({})",
                        day,
                        period.index,
                        period.triggers.join(", ")
                    );
                } else {
                    println!("    {:9}: N/A", day);
                }
            }
        }
        Err(e) => {
            println!("\nFailed to fetch pollen (API): {e}");
        }
    }

    println!();
    println!("Upcoming calendar events:");

    match fetch_next_events(
        &config.calendar_url,
        config.calendar_event_count,
        &config.timezone,
    )
    .await
    {
        Ok(events) if !events.is_empty() => {
            for (i, event) in events.iter().enumerate() {
                let day = event
                    .start
                    .map(|dt| dt.format("%a %Y-%m-%d").to_string())
                    .unwrap_or_else(|| "".to_string());
                let time = if event.all_day {
                    "all day".to_string()
                } else if let Some(dt) = event.start {
                    dt.format("%H:%M").to_string()
                } else {
                    "".to_string()
                };
                println!("{}. {} {:>8}  {}", i + 1, day, time, event.summary);
            }
        }
        _ => println!("No upcoming calendar events found."),
    }

    println!();
    println!("Inbound Shopify packages:");
    let packages = get_shopify_packages(&config);
    for (i, pkg) in packages.iter().enumerate() {
        println!("{}. {}", i + 1, pkg);
    }
}

// No longer needed (handled above)
// fn get_calendar_events(_config: &Config, n: usize) -> Vec<String> { ... }

fn get_shopify_packages(_config: &Config) -> Vec<String> {
    vec![
        "Order #1234: Shipped - Arriving May 13".to_string(),
        "Order #5678: In transit - Arriving May 15".to_string(),
    ]
}
