use serde::Deserialize;
use std::error::Error;

#[allow(dead_code)]
#[derive(Debug, Deserialize)]
struct GeocodeResponse {
    lat: f64,
    lon: f64,
    #[allow(dead_code)]
    name: String,
    #[allow(dead_code)]
    country: String,
}

// returns (lat, lon) for specified city name
pub async fn geocode_city(city: &str, api_key: &str) -> Result<(f64, f64), Box<dyn Error>> {
    let client = reqwest::Client::new();
    let geocode_url = format!(
        "https://api.openweathermap.org/geo/1.0/direct?q={}&limit=1&appid={}",
        city, api_key
    );
    let geo_resp = client.get(&geocode_url).send().await?;
    let geo_text = geo_resp.text().await?;
    let geo_parsed: Result<Vec<GeocodeResponse>, serde_json::Error> =
        serde_json::from_str(&geo_text);
    match geo_parsed {
        Ok(geo_vec) => {
            if geo_vec.is_empty() {
                return Err(format!(
                    "No geocoding result for city: {} (response: {})",
                    city, geo_text
                )
                .into());
            }
            Ok((geo_vec[0].lat, geo_vec[0].lon))
        }
        Err(e) => {
            eprintln!("Failed to decode geocoding response. Raw response: {geo_text}");
            Err(Box::new(e))
        }
    }
}
// ------------------ WEATHER -------------------------

#[derive(Debug, Deserialize)]
pub struct WeatherCurrent {
    pub temp: f64,
    pub weather: Vec<WeatherDesc>,
    pub dt: i64, // Unix timestamp
}

#[derive(Debug, Deserialize)]
pub struct WeatherDailyTemp {
    pub min: f64,
    pub max: f64,
}

#[derive(Debug, Deserialize)]
pub struct WeatherDesc {
    pub description: String,
}

#[derive(Debug, Deserialize)]
pub struct WeatherDaily {
    pub temp: WeatherDailyTemp,
    pub weather: Vec<WeatherDesc>,
}

#[derive(Debug, Deserialize)]
pub struct OneCallResult {
    pub current: WeatherCurrent,
    pub daily: Vec<WeatherDaily>,
}

pub struct WeatherSummary {
    pub temp: String,
    pub current_desc: String,
    pub high: String,
    pub low: String,
    pub daily_desc: String,
    pub obs_time_unix: i64,
}

/// Returns WeatherSummary struct
pub async fn fetch_weather(city: &str, api_key: &str) -> Result<WeatherSummary, Box<dyn Error>> {
    let (lat, lon) = geocode_city(city, api_key).await?;
    let client = reqwest::Client::new();
    // Get weather data from One Call 3.0
    let onecall_url = format!(
        "https://api.openweathermap.org/data/3.0/onecall?lat={}&lon={}&appid={}&units=imperial",
        lat, lon, api_key
    );
    let one_resp = client.get(&onecall_url).send().await?;
    let one_status = one_resp.status();
    let one_text = one_resp.text().await?;
    if !one_status.is_success() {
        return Err(format!("HTTP error {}: {}", one_status, one_text).into());
    }

    let one_parsed: OneCallResult = match serde_json::from_str(&one_text) {
        Ok(data) => data,
        Err(e) => {
            eprintln!("Failed to decode One Call weather response. Raw response: {one_text}");
            return Err(Box::new(e));
        }
    };

    // Current conditions
    let temp = format!("{:.1}°F", one_parsed.current.temp);
    let current_desc = one_parsed
        .current
        .weather
        .first()
        .map(|w| w.description.clone())
        .unwrap_or_else(|| "N/A".to_string());
    let current_dt = one_parsed.current.dt; // UNIX timestamp, UTC

    // Today's forecast is daily[0]
    let (high, low, daily_desc) = if let Some(today) = one_parsed.daily.first() {
        let high = format!("{:.1}°F", today.temp.max);
        let low = format!("{:.1}°F", today.temp.min);
        let desc = today
            .weather
            .first()
            .map(|w| w.description.clone())
            .unwrap_or_else(|| "N/A".to_string());
        (high, low, desc)
    } else {
        ("N/A".to_string(), "N/A".to_string(), "N/A".to_string())
    };

    Ok(WeatherSummary {
        temp,
        current_desc,
        high,
        low,
        daily_desc,
        obs_time_unix: current_dt,
    })
}
